//
//  MTPPSCRA.h
//  MTPPSCRA
//
//  Created by Tam Nguyen on 4/18/16.
//  Copyright © 2016 None. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MTObject.h"

@class MTPeripheral;


enum
{
    
    MAGTEKDYNAPROMINIBLE,
    MAGTEKDYNAPROMINIUART,
    MAGTEKETHERNET
    
};
typedef int MTPPSCRADeviceType __attribute__((deprecated("MTPPSCRADeviceType has been deprecated please use ConnectionType instead")));

 enum {
     Unknown = 0,
     Audio = 1,
     BluetoothBLE = 2,
     BLEEMV = 3,
     Bluetooth = 4,
     HID = 5,
     Serial = 6,
     Net = 7,
     Net_TLS12 = 8,
     Net_TLS12_Trust_All = 9
 };
typedef NSUInteger ConnectionType;

typedef enum {
    Connected = 0,
    Connecting = 1,
    Disconnecting = 2,
    Disconnected = 3,
    Unreachable = 4,
    Error = 5,
    Listening
}ConnectionState;

typedef enum
{
    DISPLAY_MESSAGE_BLANK = 0x00,
    DISPLAY_MESSAGE_APPROVED,
    DISPLAY_MESSAGE_DECLINED,
    DISPLAY_MESSAGE_CANCELLED,
    DISPLAY_MESSAGE_THANK_YOU,
    DISPLAY_MESSAGE_PIN_INVALID,
    DISPLAY_MESSAGE_PROCESSING,
    DISPLAY_MESSAGE_PLEASE_WAIT,
    DISPLAY_MESSAGE_HANDS_OFF,
    DISPLAY_MESSAGE_PIN_PAD_NOT_AVAILABLE,
    DISPLAY_MESSAGE_CALL_YOUR_BANK,
    DISPLAY_MESSAGE_CARD_ERROR,
    DISPLAY_MESSAGE_NOT_ACCEPTED,
    DISPLAY_MESSAGE_PROCESSING_ERROR,
    DISPLAY_MESSAGE_USE_CHIP_READER,
    DISPLAY_MESSAGE_REFER_TO_YOUR_PAYMENT_DEVICE,
    DISPLAY_MESSAGE_BITMAPSLOT0,
    DISPLAY_MESSAGE_BITMAPSLOT1,
    DISPLAY_MESSAGE_BITMAPSLOT2,
    DISPLAY_MESSAGE_BITMAPSLOT3,
    DISPLAY_MESSAGE_USER
}MessageID;




@protocol MTPPSCRADelegate <NSObject>
@optional

//device message
- (void) deviceNotPaired;
- (void) deviceDataAvailable;
- (void) deviceMessageChange;
- (void) deviceMessageCardStatus:(Byte)operationStatus cardStatus:(Byte)cardStatus;
- (void) deviceMessageDataChanged;
- (void) deviceMessageDisplayMessageDone:(Byte)operationStatus;
- (void) deviceMessagePinData:(Byte)operationsStatus pinData:(PinData*)pinData;
- (void) deviceMessageACKStatus:(Byte)status commandID:(Byte)commandID;
- (void) deviceMessageStatus:(Byte)status commandID:(Byte)commandID;
- (void) deviceMessageClearTextUserEntry:(Byte)operationStatus clearTextData:(NSData*)clearTextData;
- (void) deviceMessageDeviceInformation:(Byte)type data:(NSData*)data;
- (void) deviceMessageUserSelection:(Byte)operationStatus keyValue:(Byte)keyValue;
- (void) deviceMessageUserSignature:(Byte)status signature:(NSData*)signature;
- (void) deviceMessageDeviceStatus:(DeviceState*)deviceStatus;
- (void) deviceMessageKeyInformation:(Byte)keyID keyStatus:(Byte)keyStatus keyInfo:(NSData*)keyInfo;
- (void) deviceMessageUserDataEntry:(Byte)operationStatus;
- (void) deviceMessageKernelInformation:(Byte)kernelID kernelInformation:(NSData*)kernelInformation;
- (void) deviceMessageDeviceConfiguration:(NSData*)configData;
- (void) deviceMessageARQCRequest:(NSData*)ARCQData;
- (void) deviceMessageEMVData:(NSData*)emvData;
- (void) deviceMessageBINTableData:(NSData*)binData;
- (void) deviceMessageTipOrCashback:(Byte)status mode:(Byte)mode amount:(Byte*)amount tax:(Byte*)tax taxRate:(Byte*)taxRate tipOrCashBack:(Byte*)tipOrCashBack reserve:(Byte)reserve;
- (void) deviceMessageEMVTags:(NSData*)tagData;
- (void) deviceMessageCAPublicKey:(NSData*)keyData;
- (void) deviceMessageSelectedMenuItem:(Byte)status mode:(Byte)mode index:(Byte)index reserve:(Byte)reserve;
- (void) onDataReadyCompleteEvent:(NSData*)data;
- (void) onConnectionStateChanged:(ConnectionType)connectionType state:(ConnectionState) state;
- (void) onCertificateError:(NSError*)error;


@end





@interface MTPPSCRA : NSObject

+ (MTPPSCRA *)sharedInstance;



@property (nonatomic, strong) id<MTPPSCRADelegate> delegate;
#if TARGET_OS_OSX
- (NSArray*)requestDeviceList;
#endif

- (int)loadClientCertificate:(NSString*)format data:(NSData*)data password:(NSString*)string;
- (int)setCAPublicKey:(Byte)operation keyBlock:(NSData*)keyBlock;
- (int)openDevice;
- (int)deviceReset;
- (int)cancelOperation;
- (int)closeDevice;
- (BOOL)isDeviceSRED;
- (int)updateFirmware:(NSData*)firmwareData;
- (NSString*)getDeviceSerial;
- (NSString*)getPINKSN;
- (NSString*)getPINStatusCode;
- (NSString*)getEPB;
- (BOOL)isDeviceOpened;
- (BOOL)isDeviceConnected;
- (NSString*)getSDKVersion;
- (int)sendSpecialCommand:(NSData*)commandIn;
- (int)getSpecialCommand:(NSData*)commandIn;
- (int)endSession:(Byte)messageID;
- (int)sendAcquirerResponse:(NSData*)responseData;
- (int)requestTipOrCashback:(Byte)waitTime mode:(Byte)mode tone:(Byte)tone amount:(Byte*)amount taxAmount:(Byte*)taxAmount taxRate:(Byte*)taxRate tipMode:(Byte)tipMode option1:(Byte)option1 option2:(Byte)option2 option3:(Byte)option3 reserved:(Byte*)reserved;
- (int)requestSignature:(Byte)waitTime beepTones:(Byte)beepTones options:(Byte)options;
- (int)requestGetEMVTag:(Byte)tagType tagOperation:(Byte)tagOperation inputTLVData:(NSData*)inputTLVData dataBase:(Byte)dataBase option:(Byte)option reserved:(NSData*)reserved;
- (int)requestSetEMVTag:(Byte)tagType tagOperation:(Byte)tagOperation inputTLVData:(NSData*)inputTLVData dataBase:(Byte)dataBase option:(Byte)option reserved:(NSData*)reserved;
- (int)setBINTableData:(NSData*)binTable;
- (int)requestBINTableData;
- (CardDataInfo*)getCardDataInfo;
- (void)stopScanningForPeripherals;
- (void)setAddress: (NSString*)address;
- (int)setPAN: (NSString*)pan;
- (int)setAmount: (NSString*)amount;
- (int)confirmAmount:(Byte)waitTime beepTones:(Byte)beepTones;
- (void)setConnectionType: (ConnectionType)connection;
- (void)startScanningForPeripherals:(void(^)(NSArray*))handler;
- (int)setDisplayMessage: (Byte)waitTime messageID:(Byte)messageID;
- (UserDataEntry*)getUserDataEntry:(BOOL)clearBuffer;
- (int)requestPINBypass;
- (int)requestClearTextUserDataEntry:(Byte)waitTime messageID:(Byte)messageID beepTones:(Byte)beepTones;
- (int)requestDeviceInformation:(Byte)mode;
- (int)requestResponse:(Byte)waitTime selectMsg:(Byte)selectMsg keyMask:(Byte)keyMask beepTones:(Byte)beepTones;
- (int)requestDeviceStatus;
- (int)requestCard:(Byte)waitTime messageID:(Byte)messageID beepTones:(Byte)beepTones;
- (int)requestPIN:(Byte)waitTime pinMode:(Byte)pinMode maxPINLength:(Byte)maxPINLength minPINLength:(Byte)minPINLength beepTones:(Byte)beepTones option:(Byte)option;
- (int)requestKeyInformation:(Byte)information;
- (int)requestKernelInformation:(Byte)kernelInfoID;
- (int)requestUserDataEntry:(Byte)waitTime messageID:(Byte)messageID beepTones:(Byte)beepTones;
- (int)requestManualCardData:(Byte)waitTime beepTones:(Byte)beepTones options:(Byte)options;
- (int)requestDeviceConfiguration;
- (int)requestSmartCard:(Byte)cardType confirmationWaitTime:(Byte)confirmationWaitTime pinEnteringTime:(Byte)pinEnteringTime beepTones:(Byte)beepTones option:(Byte)option amount:(Byte*)amount transactionType:(Byte)transactionType cashBack:(Byte*)cashBack reserved:(NSData*)reserved;
- (int)sendBigBlockData:(Byte)command dataBlock:(NSData*)dataBlock completion:(void (^)(void))completion;
//- (BOOL)startListeningOnPort:(int)port;
- (int)getSelectedMenuItem:(Byte)waitTime mode:(Byte)mode tone:(Byte)tone;
- (NSInteger)requestDeviceInformation:(unsigned char)mode information:(NSString **)information;
@end

