//
//  MTSession.m
//  MTPPSCRASample
//
//  Created by Tam Nguyen on 8/29/17.
//  Copyright © 2017 MagTek. All rights reserved.
//

#import "MTSession.h"
#import "HexUtility.h"
#import "NSObject+TLVParser.h"

#include <CoreFoundation/CoreFoundation.h>
#include <sys/socket.h>
#include <netinet/in.h>

char const tEND = 0xC0;
char const tESC = 0xDB;
char const tTEND = 0xDC;
char const tTESC = 0xDD;

@interface MTSession()
{
    TransactionMode currentMode;
    BOOL _listenForConnections;
}
@end
@implementation MTSession

- (id)initWithMode:(TransactionMode)mode
{
    self = [super init];
    if (self)
    {
        currentMode = mode;
        [self setupSession];
    }
    return self;
}

-(id)init
{
    self = [super init];
    if (self)
    {
        [self setupSession];
    }
    return self;
}

- (void)setupSession
{
    self.ppSCRA = [[MTPPSCRA alloc]init];
    //Download neccessary cert at https://www.magtek.com/support/dynapro-go?tab=software
    NSString *identityPath = [[NSBundle mainBundle] pathForResource:@"client" ofType:@"p12"];
    NSData *PKCS12Data = [NSData dataWithContentsOfFile:identityPath];
    
    [self.ppSCRA loadClientCertificate:@"p12" data:PKCS12Data password:@"password"];
    
}
- (void)startListening
{

    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [self setUpServerListener];
    });
}

-(NSData*) toData:(unsigned char*)data
{
    NSMutableData* hexData = [[NSMutableData alloc]init];
    
    int i = 0;
    BOOL foundSlipFrame = false;
    
    do
    {
        if(data[i] == 0xc0)
            foundSlipFrame = true;
        [hexData appendBytes:&data[i] length:1];
        i++;
        
    }
    while (data[i] != 0xc0 && foundSlipFrame);
    [hexData appendBytes:&data[i] length:1];
    return hexData;
}

- (NSData *)unescapedData:(NSData*)dataIn {
    
    const char *bytes = [dataIn bytes];
    NSMutableData *unescapedData = [[NSMutableData alloc] init];
    if (dataIn) {
        // Take out all the escape sequences in the data.
        for(int i=0; i< [dataIn length]; i++){
            char currentByte = bytes[i];
            
            if(currentByte == tEND && i ==0)
                continue;
            else if(currentByte == tEND && i+1 == [dataIn length])
                break;
            else if(currentByte == tESC) {
                currentByte = bytes[++i];
                if(currentByte == tTEND)
                    currentByte = tEND;
                else if(currentByte ==tTESC)
                    currentByte = tESC;
            }
            [unescapedData appendBytes:&currentByte length:1];
        }
    }
    return unescapedData;
}

- (void)stopListening
{
    if([server isListening])
        [server stopServer];
    if([_ppSCRA isDeviceOpened])
    {
        [_ppSCRA closeDevice];
        
    }
}

- (void)setUpServerListener
{
    
    MTSocket * aSocket = nil;
    server = [[MTSocketServer alloc] initListeningOnPort:1026];
    @try {
        [server listen];
        if([self.delegate respondsToSelector:@selector(sessionDidStart:)])
        {
            [self.delegate sessionDidStart:self];
        }
        while ((aSocket = (MTSocket *)[server acceptConnection]) != nil) {
            NSLog(@"Connected from %@:%d", [aSocket remoteHost], [aSocket remotePort]);
            
            Byte response[6] = {0xC0, 0x02, 0x01, 0x00, 0x2F, 0xC0};
            [aSocket writeData:[NSData dataWithBytes:&response length:6]];
            NSData * aByte = [aSocket readData:256];
            NSData* messageData =[self unescapedData: aByte];
            unsigned char *bytePtr = (unsigned char *)[messageData bytes];
            
            NSData* hostNameData = [messageData subdataWithRange:NSMakeRange(6, messageData.length - 6)];
            NSString* hexString = [HexUtility stringFromHexString: [HexUtility toHex:hostNameData]];
            tlsEnabled = bytePtr[4];
            
            ipAddress = hexString;
            NSInteger portNumber = (int)bytePtr[2] << 8;
            portNumber |= (int)bytePtr[3];
            ipAddress = [self->ipAddress stringByAppendingFormat:@":%li", (long)portNumber];
            
            [server stopServer];
            if([self.delegate respondsToSelector:@selector(didGetNewConnection:shouldTrustAll:useTLS:)])
            {
                [self.delegate didGetNewConnection:ipAddress shouldTrustAll:_shouldTrustAll useTLS:tlsEnabled];
            }
            //[self startTransaction];
        }
    } @catch (NSException * e) {
        NSLog(@"Exception : %@", e);
    }
    
    
}
/*
 - (void)startTransaction
 {
 
 dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 2 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
 self.ppSCRA.delegate = self;
 if(self->tlsEnabled)
 {
 if(!self.shouldTrustAll)
 [self.ppSCRA setConnectionType:Net_TLS12];
 else
 [self.ppSCRA setConnectionType:Net_TLS12_Trust_All];
 }
 else
 {
 [self.ppSCRA setConnectionType:Net];
 }
 [self.ppSCRA setAddress:self->ipAddress];
 [self.ppSCRA openDevice];
 });
 }*/

- (void) onConnectionStateChanged:(ConnectionType)connectionType state:(ConnectionState)state
{
    NSLog(@"State changed to: %u", state);
    
    if([self.delegate respondsToSelector:@selector(sessionStateDidChange:instance:)])
    {
        [self.delegate sessionStateDidChange:state instance:self];
    }
    
    if(state == Connected)
    {
        if(currentMode == Restaurant)
        {
            [self.ppSCRA requestClearTextUserDataEntry:0x3c messageID:0x05 beepTones:0x01];
        }
        else
        {
            [self.ppSCRA requestClearTextUserDataEntry:0x3c messageID:0x06 beepTones:0x01];
        }
    }
}

- (void)deviceMessageClearTextUserEntry:(Byte)operationStatus clearTextData:(NSData *)clearTextData
{
    if([self.delegate respondsToSelector:@selector(didGetClearText:)])
    {
        
        NSString* text = [HexUtility stringFromHexString:[HexUtility toHex:clearTextData]];
        [self.delegate didGetClearText:text];
    }
    
    if(operationStatus == 0x00)
    {
        if(currentMode == Restaurant)
        {
            NSArray* menuItems = @[ @"Tab 1", @"Tab 2", @"Tab 3", @"Tab 4", @"Tab 5",@"Tab 6",@"Tab 7", @"Tab 8",@"Tab 9", @"Tab 10"];
            NSMutableData* menuData = [[NSMutableData alloc] init];
            Byte cr[1] = {(Byte)0x0d};
            for(NSString* item in menuItems)
            {
                //NSData* itemData = [HexUtility getBytesFromHexString:item];
                [menuData appendBytes:[item UTF8String] length:item.length];
                [menuData appendBytes:cr length:1];
                
            }
            
            [menuData appendBytes:cr length:1];
            [self.ppSCRA sendBigBlockData:0x16 dataBlock:menuData completion:^{
                [self.ppSCRA getSelectedMenuItem:0x3c mode:0x00 tone:0x01];
                
            }];
        }
        else
        {
            Byte amount[]             = {0x00, 0x00, 0x00, 0x1, 0x00, 0x00};
            Byte taxAmount[]             = {0x00, 0x00, 0x00, 0x00, 0x10, 0x00};
            Byte taxRate[]             = {0x10, 0x00, 0x00};
            
            Byte reserve[26] = {0};
            
            [self.ppSCRA requestTipOrCashback:0x3c mode:0x01 tone:0x01 amount:amount taxAmount:taxAmount taxRate:taxRate tipMode:0x00 option1:0x01 option2:0x02 option3:0x03 reserved:reserve];
            
        }
    }
}
- (void)deviceMessageSessionStart:(Byte)status
{
    if(status == 0x00)
    {
        if([self.delegate respondsToSelector:@selector(sessionDidStart:)])
        {
            [self.delegate sessionDidStart:self];
        }
        
        if(currentMode == Restaurant)
        {
            [self.ppSCRA requestClearTextUserDataEntry:0x3c messageID:0x05 beepTones:0x01];
        }
        else
        {
            [self.ppSCRA requestClearTextUserDataEntry:0x3c messageID:0x06 beepTones:0x01];
        }
    }
}

@end
