//
//  MTSocket.h
//  MTPPSCRASample
//
//  Created by Tam Nguyen on 8/29/17.
//  Copyright © 2017 MagTek. All rights reserved.
//

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>

#import <Foundation/Foundation.h>
//#import "MTSocket.h"

#define MTSocketConnectException @"MTSocketConnectException"

typedef enum {
	MTSocketStateUnopen,
	MTSocketStateOpen,
	MTSocketStateClosed
} MTSocketState;

@interface MTSocket : NSObject {
    int _fileDescriptor;
	int _port;
	NSString * _host;
	MTSocketState socketState;
}

/**
 * Creates a socket from an open file descriptor with the specified information.
 * @param fileDescriptor The file descriptor that this socket will wrap.
 * @param host The remote host to which the file descriptor is connected.
 * @param port The remote port through which the file descriptor is connected.
 * @return A new MTSocket object.
 */
- (id)initWithFileDescriptor:(int)fileDescriptor remoteHost:(NSString *)host port:(int)port;

/**
 * Creates a socket that is connected to a specified host on a specified port.
 * @param host The host to which the socket will connect itself.
 * @param port The port through which the socket will connect itself.
 * @return A new MTSocket object.
 * @throws MTSocketConnectException When the socket cannot be opened, or 
 * when the remote host is not available on the specified port.
 */
- (id)initWithRemoteHost:(NSString *)host port:(int)port;

/**
 * @return The remote address.
 */
- (NSString *)remoteHost;

/**
 * @return The remote port.
*/
- (int)remotePort;

/**
 * Gets the file descriptor.
 * @return The underlying file descriptor for the socket.
 */
- (int)fileDescriptor;
- (void)writeData:(NSData *)theData;
- (NSData *)readData:(UInt32)length;
@end
